@@ -1,7 +1,39 @@
-confint.HLfit <- function(object,parm,level=0.95,verbose=TRUE,...) {
-  if ( .REMLmess(object,return_message=FALSE)) {
-    warning("REML fits are not quite suitable for computing intervals for fixed effects.")
-  } 
+.confint.boot <- function(boot_args, object, expr_t, t_fn=NULL, parm, boot.ci, level, ...) {
+  spaMM_boot_args <- intersect(names(boot_args),names(formals(spaMM_boot)))
+  spaMM_boot_args <- boot_args[spaMM_boot_args]
+  spaMM_boot_args$object <- object
+  if (is.null(spaMM_boot_args$type)) spaMM_boot_args$type <- "residual"
+  boot.ci_args <- intersect(names(boot_args),names(formals(boot.ci)))
+  boot.ci_args <- boot_args[boot.ci_args]
+  if (is.null(t_fn)) {
+    spaMM_boot_args$simuland <- function(y, ...) {
+      upd <- update_resp(object, newresp=y)
+      eval(expr_t, list(hlfit=upd))
+    }
+    boot.ci_args$t0 <- eval(expr_t, list(hlfit=object))
+  } else {
+    spaMM_boot_args$simuland <- function(y, ...) {
+      upd <- update_resp(object, newresp=y)
+      t_fn(upd)
+    }
+    boot.ci_args$t0 <- t_fn(object, ...)
+  }
+  ts <- drop(do.call(spaMM_boot,spaMM_boot_args, ...)[["bootreps"]])
+  boot.ci_args$t <- ts
+  boot.ci_args$boot.out <- list(R = length(ts), sim="parametric")
+  boot.ci_args$conf <- level
+  if (is.null(boot.ci_args$ci_type)) {
+    boot.ci_args$type <- c("basic","perc","norm")
+  } else {
+    boot.ci_args$type <- boot.ci_args$ci_type
+    boot.ci_args$ci_type <- NULL
+  }
+  resu <- do.call("boot.ci", boot.ci_args)
+  resu$call <- match.call()
+  return(resu)
+}
+
+.confint.LRT <- function(level, parm, object, verbose) {
   dlogL <- qchisq(level,df=1)/2
   znorm <- qnorm((1+level)/2)
   if (is.character(parm)) {
@@ -36,9 +68,9 @@ confint.HLfit <- function(object,parm,level=0.95,verbose=TRUE,...) {
   #
   # FIXME test processed nature to prevent multinom stuff here
   intervalinfo <- list(fitlik=object$APHLs[[lik]],
-                     targetlik=object$APHLs[[lik]]-dlogL,
-                     parm=parm,
-                     asympto_abs_Dparm=asympto_abs_Dparm)
+                       targetlik=object$APHLs[[lik]]-dlogL,
+                       parm=parm,
+                       asympto_abs_Dparm=asympto_abs_Dparm)
   oldopt <- .options.processed(lc$processed, intervalInfo=intervalinfo, augZXy_cond=FALSE)
   if (X_is_scaled) {
     lc$processed$intervalInfo$MLparm <- .scale(beta=object$fixef,X=X.pv)[parm]
@@ -193,6 +225,32 @@ confint.HLfit <- function(object,parm,level=0.95,verbose=TRUE,...) {
   .options.processed(lc$processed, oldopt)
   interval <- c(lowerfit$fixef[parm],upperfit$fixef[parm])
   names(interval) <- paste(c("lower","upper"),parm)
-  if (verbose) print(interval)
-  invisible(list(lowerfit=lowerfit,upperfit=upperfit,interval=interval))
+  return(list(lowerfit=lowerfit,upperfit=upperfit,interval=interval))
+}
+
+
+confint.HLfit <- function(object, parm, level=0.95, verbose=TRUE, 
+                          boot_args=NULL,...) {
+  if ( .REMLmess(object,return_message=FALSE)) {
+    warning("REML fits are not quite suitable for computing intervals for fixed effects.")
+  } 
+  if (is.character(parm)) {
+    if (is.list(boot_args)) expr_t <- substitute(fixef(hlfit)[[parm]], list(parm=parm))
+    t_fn <- NULL
+  } else if (is.function(parm)) {
+    t_fn <- parm
+  } else {
+    expr_t <- parm
+    t_fn <- NULL
+    #if (is.null(boot_args)) boot_args <- list(nsim=999L) fail is the example from the doc...
+  }
+  if (  is.list(boot_args)) {
+    boot_res <- .confint.boot(boot_args, object, expr_t, t_fn, parm, boot.ci, level) 
+    if (verbose) print(boot_res)
+    invisible(boot_res)
+  } else {
+    resu <- .confint.LRT(level, parm, object, verbose)
+    if (verbose) print(resu$interval)
+    invisible(resu)
+  }
 }
